/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.demos;

import com.smardec.asc.hyperlink.HyperlinkUtil;
import com.smardec.demo.j2native.DemoableAdapter;
import com.smardec.demo.j2native.Util;
import com.smardec.demo.j2native.tools.SysInfoUtil;
import com.smardec.demo.j2native.win32.regviewer.RegViewer;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.*;
import java.io.File;
import java.util.Vector;


public class WinRegistryDemo extends DemoableAdapter {
    public final static String CONSTRAINT_WIN_REGISTRY = "WinRegistry";
    public final static String CAPTION_WIN_REGISTRY = "Win registry";
    public final static String INTRO_WIN_REGISTRY = "<html><head>" + css + "</head><body>" +
            "<p align=left color=black>This is a more interesting variant of J2Native using. Moreover, it is very useful and used frequently. Registry Viewer, as it is clear from the name, downloads data from Windows registry. (NOTE: program works in Read-Only mode. No changes will be made under any circumstances.)</p>" +
            "</body></html>";
    public final static String URL_API_REFERENCE = "registry.html";
    public final static String API_REFERENCE_PATH = "demo-resources/api-reference/RegEnumValue.html";

    public final static String SRC_WIN_REGISTRY = "<html><head>" + css + "</head><body>" +
            "<p align=left color=black>As an example lets have a look at how it is possible to write a small procedure which gets a handle to a currently open key of the registry and as a result gives you back a list of values for this registry key. And lets have a closer look at the description of <i>RegEnumValue</i> function" +
            "<p align=right color=black><a href=" + URL_API_REFERENCE + ">View API Reference (in browser)</a></p>" +
            "And now look at the code of the procedure which we will call <i>retrieveData():</i>" +
            "<br><pre>    /**\n" +
            "     * Class that holds registry info\n" +
            "     */\n" +
            "    private class RegInfo {\n" +
            "        public String valueName;\n" +
            "        public int type;\n" +
            "        public byte[] data;\n" +
            "\n" +
            "        public RegInfo(String valueName, int type, byte[] data) {\n" +
            "            this.valueName = valueName;\n" +
            "            this.type = type;\n" +
            "            this.data = data;\n" +
            "        }\n" +
            "    }" +
            "\n" +
            "    /**\n" +
            "     * This method is used to obtain the values for the specified open registry key\n" +
            "     *\n" +
            "     * @param hKey handle to key to query\n" +
            "     * @return list of RegInfo\n" +
            "     */\n" +
            "    private ArrayList retrieveData(Argument hKey) {\n" +
            "        // load Advapi32.dll libraray\n" +
            "        Library advapi32 = new Library(\"Advapi32\");\n" +
            "        // locate function\n" +
            "        Function regEnumValue = advapi32.getFunction(\"RegEnumValueA\");" +
            "\n" +
            "        // prepare arguments\n" +
            "        ArrayList list = new ArrayList();\n" +
            "        UInt lResult = new UInt();\n" +
            "        UInt dwIndex = new UInt();\n" +
            "        AnsiString valueName = new AnsiString(\"\", 512);\n" +
            "        Pointer lpValueName = new Pointer(valueName);\n" +
            "        UInt cValueName = new UInt();\n" +
            "        Pointer lpcValueName = new Pointer(cValueName);\n" +
            "        Pointer.Void lpReserved = new Pointer.Void();\n" +
            "        UInt type = new UInt();\n" +
            "        Pointer lpType = new Pointer(type);\n" +
            "        Const dwIndexConst = new Const(dwIndex);\n" +
            "        if (dataMtx == null) {\n" +
            "            dataMtx = new UInt8[1024];\n" +
            "            for (int i = 0; i < dataMtx.length; i++) {\n" +
            "                dataMtx[i] = new UInt8();\n" +
            "            }\n" +
            "        }\n" +
            "        ArrayArgument data = new ArrayArgument(dataMtx);\n" +
            "\n" +
            "        Pointer lpData = new Pointer(data);\n" +
            "        UInt cData = new UInt();\n" +
            "        Pointer lpcbData = new Pointer(cData);\n" +
            "\n" +
            "        // create a loop to retrive all values\n" +
            "        while (true) {\n" +
            "            cValueName.setValue(512);\n" +
            "            cData.setValue(1024);\n" +
            "            regEnumValue.invoke(lResult, new Argument[]{hKey, dwIndexConst, lpValueName, lpcValueName,\n" +
            "                    lpReserved, lpType, lpData, lpcbData});\n" +
            "            if (lResult.intValue() != RWConstants.ERROR_SUCCESS) {\n" +
            "                break;\n" +
            "            }\n" +
            "            // increment index\n" +
            "            dwIndex.setValue(dwIndex.intValue() + 1);\n" +
            "\n" +
            "            String valueNameStr = valueName.toString().substring(0, cValueName.intValue());\n" +
            "            byte[] bData = new byte[cData.intValue()];\n" +
            "            System.arraycopy(data.toBytesValue(), 0, bData, 0, bData.length);\n" +
            "\n" +
            "            // add obtained value to list\n" +
            "            list.add(new RegInfo(valueNameStr, type.intValue(), bData));\n" +
            "        }\n" +
            "        return list;\n" +
            "    }</pre>" +
            "<br>And again everything turned out to be very simple, it is even amazing. The procedure is being used by RegViewer program itself and nothing can prevent you from using is too and we even recommend you to do it." +
            "</p>" +
            "</body></html>";


    private JPanel jPanelOptions = null;
    private JPanel jPanelDesc = null;
    private JPanel jPanelSrc = null;

    private RegViewer regViewer;
    private boolean inProgress = false;

    public String getID() {
        return CONSTRAINT_WIN_REGISTRY;
    }

    public String getCaption() {
        return CAPTION_WIN_REGISTRY;
    }

    public Vector getContentDescriptions() {
        return null;
    }

    public int getState() {
        return STATE_NONE;
    }

    public JPanel getDemoPanel() {
        JPanel jPanelContents = new JPanel(new GridBagLayout()) {
            public void paint(Graphics g) {
                super.paint(g);
                if (!SysInfoUtil.isWin()) {
                    Font font = g.getFont().deriveFont(Font.BOLD, 20);
                    String str = "To view this demo run it under windows";
                    FontMetrics fontmetrics = getFontMetrics(font);
                    g.setFont(font);
                    g.setColor(Color.black);
                    g.drawString(str, (getWidth() - fontmetrics.stringWidth(str)) / 2, (getHeight() - fontmetrics.getHeight()) / 2);
                }
            }
        };
        TitledBorder titledBorder = BorderFactory.createTitledBorder(CAPTION_WIN_REGISTRY);
        titledBorder.setTitleColor(Color.black);
        jPanelContents.setBorder(titledBorder);

        if (SysInfoUtil.isWin()) {
            regViewer = new RegViewer();
            Util.constrain(jPanelContents, new JScrollPane(regViewer),
                    0, 0, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1, 1,
                    0, 5, 0, 5);
        }
        return jPanelContents;
    }

    public JPanel getOptionPanel() {
        if (jPanelOptions != null)
            return jPanelOptions;
        return null;
    }

    public JPanel getDescriptionPanel() {
        if (jPanelDesc != null)
            return jPanelDesc;

        jPanelDesc = new JPanel(new GridBagLayout());
        Util.constrain(jPanelDesc, new JLabel(INTRO_WIN_REGISTRY),
                0, 1, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                5, 5, 5, 5);
        Util.constrain(jPanelDesc, Box.createGlue(),
                0, 2, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                0, 0, 0, 0);
        return jPanelDesc;
    }

    public JPanel getSrcPanel() {
        if (jPanelSrc != null)
            return jPanelSrc;

        jPanelSrc = new JPanel(new GridBagLayout());
        jPanelSrc.setBorder(BorderFactory.createTitledBorder("Windows: Registry Viewer Code Sample"));

        JEditorPane jEditorPane = new JEditorPane("text/html", SRC_WIN_REGISTRY);
        jEditorPane.setEditable(false);
        jEditorPane.setCaretPosition(0);
        jEditorPane.setBackground(jPanelSrc.getBackground());
        jEditorPane.addHyperlinkListener(new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                    if (URL_API_REFERENCE.equals(e.getDescription())) {
                        try {
                            String s;
                            try {
                                s = new File(API_REFERENCE_PATH).getCanonicalFile().toURI().toString();
                            } catch (NoSuchMethodError er) {
                                s = new File(API_REFERENCE_PATH).getCanonicalFile().toURL().toString();
                                s = convertURL2URI(s);
                            }
                            HyperlinkUtil.openInternetBrowser(s);
                        } catch (Exception e1) {
                        }
                    }
                }
            }
        });

        Util.constrain(jPanelSrc, new JScrollPane(jEditorPane),
                0, 1, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                5, 5, 5, 5);
        return jPanelSrc;
    }

    public void demoWillBecomeVisible() {
        if (SysInfoUtil.isWin()) {
            if (!regViewer.initFunctions()) {
                return;
            }
            if (inProgress) {
                return;
            }
            inProgress = true;
            new Thread(new Runnable() {
                public void run() {
                    Cursor oldCursor = parentFrame.getCursor();
                    parentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                    regViewer.loadRegistry();
                    parentFrame.setCursor(oldCursor);
                    inProgress = false;
                }
            }).start();
        }
    }
}

