/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.tools;

import com.smardec.j2native.Library;
import com.smardec.j2native.LibraryNotFoundException;

import java.util.HashMap;
import java.util.Iterator;


public class LibraryManager {
    private static LibraryManager instance;
    private HashMap libMap;

    public static final LibraryManager getInstance() {
        if (instance == null) {
            instance = new LibraryManager();
        }
        return instance;
    }

    private LibraryManager() {
        this.libMap = new HashMap();
    }

    public boolean registerLibrary(String customName, Library library) throws LibraryNotFoundException {
        if (!libMap.containsKey(customName)) {
            library.load();
            libMap.put(customName, library);
            return true;
        }
        return false;
    }

    public boolean registerLibrary(Library library) throws LibraryNotFoundException {
        if (!libMap.containsKey(library.getName())) {
            library.load();
            libMap.put(library.getName(), library);
            return true;
        }
        return false;
    }

    public boolean unregisterLibrary(String libName) {
        Library library = (Library) libMap.remove(libName);
        if (library != null) {
            library.unload();
            return true;
        }
        return false;
    }

    public Library getLibrary(String libName) {
        return (Library) libMap.get(libName);
    }

    public void reset() {
        Iterator iterator = libMap.keySet().iterator();
        while (iterator.hasNext()) {
            Library library = (Library) libMap.get(iterator.next());
            library.unload();
        }
        libMap.clear();
    }

}





















